import os
import json
from datetime import datetime
from modules.constants import HISTORY_FILE, PSVMP_DIR
from modules.helpers import logger

def log_to_history(url, media_type, status="completed", error_message=None):
    try:
        os.makedirs(PSVMP_DIR, exist_ok=True)
        
        # Convert exception objects to strings for JSON serialization
        if error_message and isinstance(error_message, Exception):
            error_message = str(error_message)
        
        entry = {
            'timestamp': datetime.now().isoformat(),
            'url': url,
            'media_type': media_type,
            'status': status,
            'error': error_message
        }
        
        with open(HISTORY_FILE, 'a', encoding='utf-8') as f:
            f.write(json.dumps(entry, ensure_ascii=False) + '\n')
        
        logger.info(f"Logged to history: {url} ({media_type}) - {status}")
        
    except Exception as e:
        logger.error(f"Failed to write to history file: {e}")

def read_history(limit=None):
    try:
        if not os.path.exists(HISTORY_FILE):
            return []
        
        with open(HISTORY_FILE, 'r', encoding='utf-8') as f:
            lines = f.readlines()
        
        entries = []
        for line in lines:
            try:
                entries.append(json.loads(line.strip()))
            except json.JSONDecodeError:
                continue
        
        entries.reverse()
        
        if limit:
            entries = entries[:limit]
        
        return entries
        
    except Exception as e:
        logger.error(f"Failed to read history file: {e}")
        return []

def clear_history():
    try:
        if os.path.exists(HISTORY_FILE):
            os.remove(HISTORY_FILE)
            logger.info("History file cleared")
            return True
        return False
    except Exception as e:
        logger.error(f"Failed to clear history file: {e}")
        return False

def show_history(limit=10):
    entries = read_history(limit)
    
    if not entries:
        print("No history entries found.")
        return
    
    print(f"\nRecent History (last {len(entries)} entries):")
    print("=" * 80)
    
    for i, entry in enumerate(entries, 1):
        timestamp = datetime.fromisoformat(entry['timestamp']).strftime('%Y-%m-%d %H:%M:%S')
        status_icon = "✓" if entry['status'] == 'completed' else "✗"
        
        print(f"{i}. [{timestamp}] {status_icon} {entry['media_type'].upper()}")
        print(f"   URL: {entry['url']}")
        if entry['status'] != 'completed' and entry.get('error'):
            print(f"   Error: {entry['error']}")
        print()


import os
import json
import platform
import logging
from modules.constants import (
    DEFAULT_CONFIG, TEMP_FOLDER, CONVERTED_FOLDER, LOG_FOLDER,
    DEFAULT_VITA_IP, DEFAULT_VITA_PORT, VITA_VIDEO_PATH, VITA_MUSIC_PATH,
    MAX_RETRIES, RETRY_DELAY, PSVMP_DIR
)
from modules.helpers import logger

def get_config_path():
    return os.path.join(PSVMP_DIR, 'configuration.json')

def load_config(silent=False):
    config_path = get_config_path()
    
    if os.path.exists(config_path):
        try:
            with open(config_path, 'r', encoding='utf-8') as f:
                loaded_config = json.load(f)
            
            # global variables
            global DEFAULT_VITA_IP, DEFAULT_VITA_PORT, VITA_VIDEO_PATH, VITA_MUSIC_PATH, MAX_RETRIES, RETRY_DELAY
            
            DEFAULT_VITA_IP = loaded_config.get('vita_ip', DEFAULT_CONFIG['vita_ip'])
            DEFAULT_VITA_PORT = loaded_config.get('vita_port', DEFAULT_CONFIG['vita_port'])
            VITA_VIDEO_PATH = loaded_config.get('video_path', DEFAULT_CONFIG['video_path'])
            VITA_MUSIC_PATH = loaded_config.get('music_path', DEFAULT_CONFIG['music_path'])
            MAX_RETRIES = loaded_config.get('max_retries', DEFAULT_CONFIG['max_retries'])
            RETRY_DELAY = loaded_config.get('retry_delay', DEFAULT_CONFIG['retry_delay'])
            
            if not silent:
                logger.info(f"Configuration loaded from: {config_path}")
            return loaded_config
        except Exception as e:
            if not silent:
                logger.warning(f"Failed to load configuration: {e}. Using defaults.")
    else:
        if not silent:
            logger.info("No configuration file found. Using defaults.")
    
    return DEFAULT_CONFIG.copy()

def save_config(config=None):
    if config is None:
        config = {
            'vita_ip': DEFAULT_VITA_IP,
            'vita_port': DEFAULT_VITA_PORT,
            'video_path': VITA_VIDEO_PATH,
            'music_path': VITA_MUSIC_PATH,
            'max_retries': MAX_RETRIES,
            'retry_delay': RETRY_DELAY
        }
    
    config_path = get_config_path()
    
    try:
        with open(config_path, 'w', encoding='utf-8') as f:
            json.dump(config, f, indent=4, ensure_ascii=False)
        
        logger.info(f"Configuration saved to: {config_path}")
        return config_path
    except Exception as e:
        logger.error(f"Failed to save configuration: {e}")
        return None

def show_config():
    config = {
        'vita_ip': DEFAULT_VITA_IP,
        'vita_port': DEFAULT_VITA_PORT,
        'video_path': VITA_VIDEO_PATH,
        'music_path': VITA_MUSIC_PATH,
        'max_retries': MAX_RETRIES,
        'retry_delay': RETRY_DELAY
    }
    
    print("\nCurrent Configuration:")
    print("-" * 40)
    for key, value in config.items():
        print(f"  {key}: {value}")
    print("-" * 40)
    print(f"Config file: {get_config_path()}")

def update_config_from_args(args):
    config_changed = False
    config = {
        'vita_ip': DEFAULT_VITA_IP,
        'vita_port': DEFAULT_VITA_PORT,
        'video_path': VITA_VIDEO_PATH,
        'music_path': VITA_MUSIC_PATH,
        'max_retries': MAX_RETRIES,
        'retry_delay': RETRY_DELAY
    }
    
    # Update config with command line values
    if hasattr(args, 'ip') and args.ip != DEFAULT_VITA_IP:
        config['vita_ip'] = args.ip
        config_changed = True
    
    if hasattr(args, 'port') and args.port != DEFAULT_VITA_PORT:
        config['vita_port'] = args.port
        config_changed = True
    
    return config, config_changed

def handle_config_command(args):
    config = load_config(silent=True)
    config_path = get_config_path()
    
    if args.set_config:
        
        for setting in args.set_config:
            try:
                key, value = setting.split('=', 1)
                key = key.strip()
                
                # Convert value to appropriate type
                if key in ['vita_port', 'max_retries']:
                    value = int(value)
                elif key in ['retry_delay']:
                    value = float(value)
                elif key in ['vita_ip', 'video_path', 'music_path']:
                    value = str(value)
                
                if key in config:
                    old_value = config[key]
                    config[key] = value
                    print(f"Updated {key}: {old_value} -> {value}")
                    logger.info(f"Config updated: {key} = {value}")
                else:
                    print(f"Warning: Unknown configuration key '{key}'")
            except ValueError:
                print(f"Error: Invalid format for '{setting}'. Use key=value format.")
                return False
        
        saved_path = save_config(config)
        if saved_path:
            print(f"\nConfiguration saved to: {saved_path}")
            return True
        else:
            print("Error: Failed to save configuration")
            return False
    
    elif args.show_config:
        show_config()
        return True
    
    else:
        print(f"\nConfiguration file location: {config_path}")
        show_config()
        print("\nUse --config set key=value to change settings")
        print("Example: --config set vita_ip=192.168.1.10 --config set vita_port=1338")
        return True


import requests
from packaging import version

try:
    from modules.VERSION import VERSION
except ImportError:
    from VERSION import VERSION

PACKAGE_NAME = "psvmp"
PYPI_URL = f"https://pypi.org/pypi/{PACKAGE_NAME}/json"

def check_for_update():
    current = str(VERSION)
    try:
        resp = requests.get(PYPI_URL, timeout=10)
        resp.raise_for_status()
        data = resp.json()
        latest = data["info"]["version"]

        if version.parse(current) < version.parse(latest):
            return True, latest  # Update available
        return False, latest  # Up to date
    except requests.exceptions.RequestException as e:
        return None, f"Network error: {e}"
    except Exception as e:
        return None, str(e)


#!/usr/bin/env python3
# https://github.com/v7upSln

import sys, io

sys.stdout = io.TextIOWrapper(sys.stdout.detach(), encoding="utf-8", errors="replace")
sys.stderr = io.TextIOWrapper(sys.stderr.detach(), encoding="utf-8", errors="replace")

logo = """
 ######   #####  #     # #     # ######  
 #     # #     # #     # ##   ## #     # 
 #     # #       #     # # # # # #     # 
 ######   #####  #     # #  #  # ######  
 #             #  #   #  #     # #       
 #       #     #   # #   #     # #       
 #        #####     #    #     # #       
"""

# Change this import
from .modules.VERSION import VERSION

import time
import os
import sys
import argparse
import logging
from datetime import datetime

# Change all module imports to relative
from .modules.constants import (
    DEFAULT_VITA_IP, DEFAULT_VITA_PORT, 
    VITA_VIDEO_PATH, VITA_MUSIC_PATH,
    TEMP_FOLDER, CONVERTED_FOLDER
)
from .modules.config import (
    load_config, save_config, show_config, 
    update_config_from_args, handle_config_command
)
from .modules.download import download_media
from .modules.conversion import convert_for_vita_video, convert_for_vita_music
from .modules.transfer import VitaFTP
from .modules.helpers import (
    setup_logging, logger, check_dependencies,
    sanitize_filename, cleanup_temp_files
)

from .modules.history import log_to_history, show_history, clear_history

from .modules.updater import check_for_update

def process_media(url, vita_ip, vita_port, media_type='video'):
    try:
        # Check dependencies first
        if not check_dependencies():
            log_to_history(url, media_type, "failed", "Missing dependencies")
            return False
        
        # 1. Download media
        logger.info(f"Starting media processing: {media_type} from {url}")
        print("=" * 50)
        print("STEP 1: DOWNLOADING MEDIA")
        print("=" * 50)
        downloaded_file = download_media(url, media_type)
        logger.info("Download completed successfully!")
        print("Download completed successfully!")
        
        # 2. Convert media
        print("\n" + "=" * 50)
        print("STEP 2: CONVERTING FOR PS VITA")
        print("=" * 50)
        
        if media_type == 'music':
            output_extension = ".mp3"
            vita_path = VITA_MUSIC_PATH
            conversion_func = convert_for_vita_music
        else:
            output_extension = "_psvita.mp4"
            vita_path = VITA_VIDEO_PATH
            conversion_func = convert_for_vita_video
        
        base_name = sanitize_filename(os.path.splitext(os.path.basename(downloaded_file))[0])
        output_filename = base_name + output_extension
        output_path = os.path.join(CONVERTED_FOLDER, output_filename)
        
        converted_file = conversion_func(downloaded_file, output_path)
        
        # 3. Transfer to Vita
        print("\n" + "=" * 50)
        print("STEP 3: TRANSFERRING TO PS VITA")
        print("=" * 50)
        print(f"Target: {vita_path}")
        print("Make sure VitaShell FTP is running (Press SELECT in VitaShell)")
        print("Waiting 3 seconds for you to confirm...")
        time.sleep(3)
        
        ftp = VitaFTP(vita_ip, vita_port)
        remote_path = f"{vita_path}{os.path.basename(converted_file)}"
        
        def progress_callback(message):
            print(f"  {message}" , flush=True)
        
        if ftp.transfer(converted_file, remote_path, progress_callback):
            logger.info(f"Media processing completed successfully: {os.path.basename(converted_file)}")
            print("\n" + "=" * 50)
            print(f"SUCCESS! {media_type.upper()} TRANSFERRED TO PS VITA")
            print("=" * 50)
            
            log_to_history(url, media_type, "completed")
            
            # Clean up
            print("\nCleaning up temporary files...")
            os.remove(downloaded_file)
            logger.info(f"Deleted temporary download: {os.path.basename(downloaded_file)}")
            print(f"Deleted temporary download: {os.path.basename(downloaded_file)}")
            
            keep_converted = input("Keep converted file for backup? (y/n): ").strip().lower()
            if keep_converted != 'y':
                os.remove(converted_file)
                logger.info(f"Deleted converted file: {os.path.basename(converted_file)}")
                print(f"Deleted converted file: {os.path.basename(converted_file)}")
            else:
                logger.info(f"Converted file kept at: {converted_file}")
                print(f"Converted file kept at: {converted_file}")
            
            return True
        
    except Exception as e:
        logger.error(f"Media processing failed: {str(e)}")
        print(f"\nERROR: {str(e)}", file=sys.stderr)
        
        log_to_history(url, media_type, "failed", str(e))
        
        # Clean up on error
        cleanup_temp_files(TEMP_FOLDER)
        temp_files = []
        if 'downloaded_file' in locals() and os.path.exists(downloaded_file):
            temp_files.append(downloaded_file)
        if 'converted_file' in locals() and os.path.exists(converted_file):
            temp_files.append(converted_file)
        
        if temp_files:
            cleanup = input("Clean up temporary files? (y/n): ").strip().lower()
            if cleanup == 'y':
                for file in temp_files:
                    try:
                        os.remove(file)
                        logger.info(f"Cleaned up on error: {os.path.basename(file)}")
                        print(f"Deleted: {os.path.basename(file)}")
                    except Exception as cleanup_error:
                        logger.warning(f"Failed to clean up {file}: {cleanup_error}")
        
        return False

def check_and_display_update_info():
    print(f"PS Vita Media Processor Version {VERSION}")
    
    update_result = check_for_update()
    
    if update_result[0] is True:
        # Update available
        print(f"Update available: Version {update_result[1]} is available!")
        print(f"Current version: {VERSION}")
        print("To update, run: pip install --upgrade psvmp")
    elif update_result[0] is False:
        # Up to date
        print(f"You have the latest version ({VERSION})")
    else:
        # Error
        print(f"Could not check for updates: {update_result[1]}")

def main():
    parser = argparse.ArgumentParser(description='PS Vita Media Processor')
    parser.add_argument('url', nargs='?', help='URL of the media file (Mega.nz, YouTube, SoundCloud, etc.)')
    parser.add_argument('--type', choices=['video', 'music'], default='video', help='Type of media to process (default: video)')
    parser.add_argument('--ip', default=DEFAULT_VITA_IP,  help=f'PS Vita IP address (default: {DEFAULT_VITA_IP})')
    parser.add_argument('--port', type=int, default=DEFAULT_VITA_PORT,  help=f'PS Vita FTP port (default: {DEFAULT_VITA_PORT})')
    parser.add_argument('--check-deps', action='store_true', help='Check if required dependencies are installed')
    parser.add_argument('-v', '--version', action='store_true', help='Show version information and exit')
    parser.add_argument('-u', '--update', action='store_true', help='Check for updates and exit')
    parser.add_argument('--history', action='store_true', help='Show download history')
    parser.add_argument('--history-clear', action='store_true', help='Clear download history')
    parser.add_argument('--history-limit', type=int, default=10, help='Number of history entries to show (default: 10)')
    
    config_group = parser.add_argument_group('configuration options')
    config_group.add_argument('--config', '-c', action='store_true', help='Show configuration file location and current settings')
    config_group.add_argument('--config-set', dest='set_config', action='append', metavar='KEY=VALUE', help='Set configuration value (can be used multiple times)')
    config_group.add_argument('--config-show', dest='show_config', action='store_true', help='Show current configuration')
    
    args = parser.parse_args()
    
    if args.update:
        check_and_display_update_info()
        sys.exit(0)
    
    if args.config or args.set_config or args.show_config:
        if handle_config_command(args):
            sys.exit(0)
        else:
            sys.exit(1)
    
    load_config(silent=True)
    
    if args.history:
        show_history(args.history_limit)
        sys.exit(0)
    
    if args.history_clear:
        if clear_history():
            print("History cleared successfully.")
        else:
            print("Failed to clear history or history was already empty.")
        sys.exit(0)
    
    if args.check_deps:
        if check_dependencies():
            print("All required dependencies are installed!")
        sys.exit(0)
    
    if args.version:
        check_and_display_update_info()
        sys.exit(0)

    if not args.url:
        parser.error("URL is required unless using --check-deps or --config")
    
    config, config_changed = update_config_from_args(args)
    if config_changed:
        logger.info("Using command-line overrides for this session")
    
    print(logo)
    print(f"    PS Vita Media Processor")
    print(f"        Version {VERSION}")
    print("-" * 50)
    print(f"Media Type: {args.type.upper()}")
    print(f"Vita IP: {args.ip}:{args.port}")
    print(f"URL: {args.url}")
    if args.type == 'video':
        print(f"Destination: {VITA_VIDEO_PATH}")
    else:
        print(f"Destination: {VITA_MUSIC_PATH}")
    print("-" * 50)
    
    if not process_media(args.url, args.ip, args.port, args.type):
        sys.exit(1)

if __name__ == "__main__":
    main()


import os

# Default configuration
DEFAULT_CONFIG = {
    "vita_ip": "192.168.1.7",
    "vita_port": 1337,
    "video_path": "ux0:/video/shows/",
    "music_path": "ux0:/music/",
    "max_retries": 5,
    "retry_delay": 3
}

USER_DOCS = os.path.join(os.path.expanduser("~"), "Documents")
BASE_DOCS_DIR = os.path.join(USER_DOCS, "PSvita media processer")

TEMP_FOLDER = os.path.join(BASE_DOCS_DIR, "temp")
CONVERTED_FOLDER = os.path.join(BASE_DOCS_DIR, "converted")

# AppData/Local/PSVMP   on Windows
if os.name == 'nt':  # Windows
    PSVMP_DIR = os.path.join(os.environ.get('LOCALAPPDATA', os.path.expanduser('~')), 'PSVMP')
else:  # Linux/macOS
    PSVMP_DIR = os.path.join(os.path.expanduser('~'), '.local', 'share', 'PSVMP')

LOG_FOLDER = os.path.join(PSVMP_DIR, "logs")
HISTORY_FILE = os.path.join(PSVMP_DIR, "history.log")

DEFAULT_VITA_IP = DEFAULT_CONFIG["vita_ip"]
DEFAULT_VITA_PORT = DEFAULT_CONFIG["vita_port"]
VITA_VIDEO_PATH = DEFAULT_CONFIG["video_path"]
VITA_MUSIC_PATH = DEFAULT_CONFIG["music_path"]
MAX_RETRIES = DEFAULT_CONFIG["max_retries"]
RETRY_DELAY = DEFAULT_CONFIG["retry_delay"]

os.makedirs(TEMP_FOLDER, exist_ok=True)
os.makedirs(CONVERTED_FOLDER, exist_ok=True)
os.makedirs(LOG_FOLDER, exist_ok=True)
os.makedirs(PSVMP_DIR, exist_ok=True)


import os
import ftplib
import time
import sys
from tqdm import tqdm
from modules.helpers import logger
from modules.constants import MAX_RETRIES, RETRY_DELAY

class VitaFTP:
    def __init__(self, ip, port):
        self.ip = ip
        self.port = port
        self.timeout = 10
        logger.info(f"Initialized FTP connection to {ip}:{port}")

    def transfer(self, local_path, remote_path, progress_callback=None):
        file_size = os.path.getsize(local_path)
        filename = os.path.basename(local_path)
        logger.info(f"Starting FTP transfer: {filename} ({file_size} bytes)")
        
        for attempt in range(1, MAX_RETRIES + 1):
            try:
                if progress_callback:
                    progress_callback(f"Attempt {attempt}/{MAX_RETRIES}: Connecting to {self.ip}:{self.port}...")
                    sys.stdout.flush()  # Force immediate output
                
                with ftplib.FTP(timeout=self.timeout) as ftp:
                    ftp.connect(self.ip, self.port)
                    
                    if '/music/' in remote_path:
                        # Music path
                        try:
                            ftp.cwd('ux0:')
                            ftp.cwd('music')
                        except ftplib.error_perm:
                            if progress_callback:
                                progress_callback("Music directory not found, creating it...")
                                sys.stdout.flush()
                            try:
                                ftp.cwd('ux0:')
                                ftp.mkd('music')
                                ftp.cwd('music')
                            except ftplib.error_perm as e:
                                if progress_callback:
                                    progress_callback(f"Directory creation error: {str(e)}")
                                    sys.stdout.flush()
                                pass
                    else:
                        # Video path
                        try:
                            ftp.cwd('ux0:')
                            ftp.cwd('video')
                            ftp.cwd('shows')
                        except ftplib.error_perm:
                            if progress_callback:
                                progress_callback("Shows directory not found, creating it...")
                                sys.stdout.flush()
                            try:
                                ftp.cwd('ux0:')
                                ftp.cwd('video')
                                ftp.mkd('shows')
                                ftp.cwd('shows')
                            except ftplib.error_perm as e:
                                if progress_callback:
                                    progress_callback(f"Directory creation error: {str(e)}")
                                    sys.stdout.flush()
                                pass
                    
                    if progress_callback:
                        progress_callback(f"Connected! Transferring {filename}...")
                        sys.stdout.flush()
                    
                    with open(local_path, 'rb') as f:
                        with tqdm(total=file_size, unit='B', unit_scale=True, 
                                 desc="Transfer Progress", leave=False) as pbar:
                            def callback(data):
                                pbar.update(len(data))
                            
                            remote_filename = os.path.basename(remote_path)
                            ftp.storbinary(f"STOR {remote_filename}", f, callback=callback)
                            print()  # Add newline after progress bar
                    
                    logger.info(f"FTP transfer completed: {filename}")
                    if progress_callback:
                        progress_callback("Transfer completed successfully")
                        sys.stdout.flush()
                    return True
                    
            except Exception as e:
                error_msg = str(e)
                logger.warning(f"FTP transfer attempt {attempt} failed: {error_msg}")
                
                if progress_callback:
                    # Show specific error message
                    if "10060" in error_msg or "timeout" in error_msg.lower():
                        progress_callback(f"[!] Connection timeout - Vita not responding")
                    elif "10061" in error_msg:
                        progress_callback(f"[!] Connection refused - FTP server not running")
                    else:
                        progress_callback(f"[!] Connection failed: {error_msg}")
                    sys.stdout.flush()
                
                if attempt < MAX_RETRIES:
                    if progress_callback:
                        progress_callback(f"[*] Retrying in {RETRY_DELAY}s... (Attempt {attempt + 1}/{MAX_RETRIES})")
                        sys.stdout.flush()
                    time.sleep(RETRY_DELAY)
                else:
                    logger.error(f"FTP transfer failed after {MAX_RETRIES} attempts: {error_msg}")
                    if progress_callback:
                        progress_callback(f"[!] Failed after {MAX_RETRIES} attempts: {error_msg}")
                        sys.stdout.flush()
                    raise Exception(f"Failed after {MAX_RETRIES} attempts: {error_msg}")
        return False


import os
import subprocess
import json
import re
from modules.helpers import logger, verify_media_file
from modules.constants import CONVERTED_FOLDER 

def run_ffmpeg_conversion(cmd, input_file, output_file, media_type):
    try:
        logger.info(f"Running FFmpeg conversion: {os.path.basename(input_file)} -> {os.path.basename(output_file)}")
        print("Running FFmpeg conversion...")
        print("Please wait, this may take a few minutes...")
        
        process = subprocess.Popen(
            cmd, 
            stdout=subprocess.PIPE, 
            stderr=subprocess.STDOUT, 
            universal_newlines=True,
            encoding='utf-8',
            errors='replace'
        )
        
        last_time = ""
        for line in process.stdout:
            if 'time=' in line:
                try:
                    # Extract just the time part
                    time_match = re.search(r'time=(\S+)', line)
                    if time_match:
                        current_time = time_match.group(1)
                        if current_time != last_time:
                            print(f"Converting... time={current_time}", flush=True)
                            last_time = current_time
                except:
                    print("Converting...", flush=True)
            elif 'error' in line.lower() or 'failed' in line.lower():
                logger.warning(f"FFmpeg warning: {line.strip()}")
                print(f"Warning: {line.strip()}", flush=True)
        
        process.wait()
        
        if process.returncode != 0:
            raise subprocess.CalledProcessError(process.returncode, cmd)
        
        if not verify_media_file(output_file, media_type):
            raise Exception("Conversion failed - output file is invalid")
        
        logger.info(f"Conversion completed: {os.path.basename(output_file)}")
        print("=" * 50, flush=True)
        print("CONVERSION COMPLETED SUCCESSFULLY!", flush=True)
        print(f"Output file: {os.path.basename(output_file)}", flush=True)
        print(f"File size: {os.path.getsize(output_file) / (1024*1024):.1f} MB", flush=True)
        print("=" * 50, flush=True)
        
        return output_file
        
    except subprocess.CalledProcessError as e:
        logger.error(f"Conversion failed with error code {e.returncode}")
        raise Exception(f"Conversion failed with error code {e.returncode}")

def convert_for_vita_video(input_file, output_file):
    logger.info(f"Converting video for PS Vita: {os.path.basename(input_file)}")
    print("Converting video for PS Vita...")
    
    if not verify_media_file(input_file, 'video'):
        raise Exception("Input video file is corrupted and cannot be converted")
    
    cmd = [
        'ffmpeg',
        '-i', input_file,
        '-c:v', 'libx264',
        '-profile:v', 'baseline',
        '-level:v', '3.1',
        '-vf', 'scale=960:544:force_original_aspect_ratio=decrease,pad=960:544:-1:-1:black',
        '-pix_fmt', 'yuv420p',
        '-b:v', '1500k',
        '-maxrate', '2000k',
        '-bufsize', '4000k',
        '-c:a', 'aac',
        '-b:a', '128k',
        '-ar', '44100',
        '-movflags', '+faststart',
        '-y',
        output_file
    ]
    
    return run_ffmpeg_conversion(cmd, input_file, output_file, 'video')

def embed_metadata_with_ffmpeg(input_file, metadata):
    if not metadata:
        return input_file
    
    logger.info("Embedding metadata into audio file...")
    print("Embedding metadata into audio file...")
    
    base_name = os.path.splitext(input_file)[0]
    output_file = f"{base_name}_tagged.mp3"
    
    cmd = [
        'ffmpeg',
        '-i', input_file,
        '-c', 'copy',
        '-y'
    ]
    
    # Add metadata fields
    if metadata.get('title'):
        cmd.extend(['-metadata', f'title={metadata["title"]}'])
    if metadata.get('artist'):
        cmd.extend(['-metadata', f'artist={metadata["artist"]}'])
    if metadata.get('album'):
        cmd.extend(['-metadata', f'album={metadata["album"]}'])
    if metadata.get('year'):
        cmd.extend(['-metadata', f'date={metadata["year"]}'])
    if metadata.get('genre'):
        cmd.extend(['-metadata', f'genre={metadata["genre"]}'])
    
    cmd.append(output_file)
    
    try:
        result = subprocess.run(cmd, capture_output=True, text=True, encoding='utf-8', errors='replace')
        if result.returncode == 0:
            os.remove(input_file)
            os.rename(output_file, input_file)
            logger.info("Metadata embedded successfully")
            print("Metadata embedded successfully")
        else:
            logger.warning(f"Could not embed metadata: {result.stderr}")
            print(f"Warning: Could not embed metadata: {result.stderr}")
            if os.path.exists(output_file):
                os.remove(output_file)
    except Exception as e:
        logger.warning(f"Metadata embedding failed: {e}")
        print(f"Warning: Metadata embedding failed: {e}")
        # Clean up failed output file
        if os.path.exists(output_file):
            os.remove(output_file)
    
    return input_file

def convert_for_vita_music(input_file, output_file):
    logger.info(f"Converting audio to MP3 for PS Vita: {os.path.basename(input_file)}")
    print("Converting audio to MP3 for PS Vita...")
    
    if not verify_media_file(input_file, 'audio'):
        raise Exception("Input audio file is corrupted and cannot be converted")
    
    # First, extract existing metadata from the file
    existing_metadata = extract_metadata_from_file(input_file)
    
    cmd = [
        'ffmpeg',
        '-i', input_file,
        '-c:a', 'mp3',
        '-b:a', '320k',  # High quality MP3
        '-ar', '44100',
        '-map_metadata', '0',  # Copy all metadata from input
        '-id3v2_version', '3',  # Use ID3v2.3 for better compatibility
        '-y',
        output_file
    ]
    
    # Add any additional metadata if we have it
    if existing_metadata:
        if existing_metadata.get('title'):
            cmd.extend(['-metadata', f'title={existing_metadata["title"]}'])
        if existing_metadata.get('artist'):
            cmd.extend(['-metadata', f'artist={existing_metadata["artist"]}'])
        if existing_metadata.get('album'):
            cmd.extend(['-metadata', f'album={existing_metadata["album"]}'])
        if existing_metadata.get('date'):
            cmd.extend(['-metadata', f'date={existing_metadata["date"]}'])
        if existing_metadata.get('genre'):
            cmd.extend(['-metadata', f'genre={existing_metadata["genre"]}'])
    
    return run_ffmpeg_conversion(cmd, input_file, output_file, 'audio')

def extract_metadata_from_file(file_path):
    try:
        cmd = [
            'ffprobe',
            '-v', 'quiet',
            '-print_format', 'json',
            '-show_format',
            file_path
        ]
        
        result = subprocess.run(cmd, capture_output=True, text=True)
        if result.returncode == 0:
            data = json.loads(result.stdout)
            format_info = data.get('format', {})
            tags = format_info.get('tags', {})
            
            normalized_tags = {}
            for key, value in tags.items():
                normalized_tags[key.lower()] = value
            
            return {
                'title': normalized_tags.get('title', ''),
                'artist': normalized_tags.get('artist', ''),
                'album': normalized_tags.get('album', ''),
                'date': normalized_tags.get('date', ''),
                'genre': normalized_tags.get('genre', '')
            }
    except Exception as e:
        logger.warning(f"Could not extract metadata from file: {e}")
        print(f"Could not extract metadata from file: {e}")
    
    return None


import os
import sys
import shutil
import subprocess
import re
import glob
import logging
from datetime import datetime
from logging.handlers import RotatingFileHandler
from modules.constants import LOG_FOLDER, TEMP_FOLDER, CONVERTED_FOLDER

# Setup logging
def setup_logging():
    log_filename = "psvmp.log"
    log_filepath = os.path.join(LOG_FOLDER, log_filename)
    
    for handler in logging.root.handlers[:]:
        logging.root.removeHandler(handler)
    
    file_handler = RotatingFileHandler(
        log_filepath,
        maxBytes=5*1024*1024,  # 5MB
        backupCount=3,          # keep 3 backup files
        encoding='utf-8'
    )
    
    # Config logging format
    formatter = logging.Formatter(
        '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    )
    file_handler.setFormatter(formatter)
    
    # Config root logger
    logging.basicConfig(
        level=logging.INFO,
        handlers=[file_handler],
        force=True
    )
    
    logger = logging.getLogger(__name__)
    logger.info("=" * 50)
    logger.info(f"PS Vita Media Processor started")
    logger.info(f"Log file: {log_filepath}")
    logger.info("=" * 50)
    
    return logger

logger = setup_logging()

def sanitize_filename(filename):
    if not filename:
        return "unknown"
    
    # Replace problematic characters
    filename = re.sub(r'[<>:"/\\|?*]', '_', filename)
    filename = re.sub(r'[\x00-\x1f\x7f-\x9f]', '', filename)
    
    # Limit length to avoid filesystem issues
    if len(filename) > 100:
        # Try to preserve the extension
        name, ext = os.path.splitext(filename)
        filename = name[:100-len(ext)] + ext
    
    return filename.strip()

def create_folders():
    os.makedirs(TEMP_FOLDER, exist_ok=True)
    os.makedirs(CONVERTED_FOLDER, exist_ok=True)

def cleanup_temp_files(folder_path):
    patterns = ['*.part', '*.ytdl', '*.part-*', '*.temp']
    for pattern in patterns:
        files = glob.glob(os.path.join(folder_path, pattern))
        for file in files:
            try:
                os.remove(file)
                logger.info(f"Cleaned up: {os.path.basename(file)}")
            except Exception as e:
                logger.warning(f"Failed to clean up {file}: {e}")

def check_dependencies():
    missing_tools = []
    
    # Check for megatools (either megatools-dl or megatools)
    if not shutil.which('megatools-dl') and not shutil.which('megatools'):
        missing_tools.append('megatools')
    
    # Check for yt-dlp
    if not shutil.which('yt-dlp'):
        missing_tools.append('yt-dlp')
    
    # Check for ffmpeg and ffprobe
    for tool in ['ffmpeg', 'ffprobe']:
        if not shutil.which(tool):
            missing_tools.append(tool)
    
    if missing_tools:
        logger.error(f"Missing required tools: {', '.join(missing_tools)}")
        print(f"Error: Missing required tools: {', '.join(missing_tools)}")
        print("\nInstall instructions:")
        print("- megatools: https://megatools.megous.com/ or package manager")
        print("- yt-dlp: pip install yt-dlp")
        print("- ffmpeg: https://ffmpeg.org/ or package manager")
        return False
    return True

def detect_url_type(url):
    url_lower = url.lower()
    
    if 'mega.nz' in url_lower or 'mega.co.nz' in url_lower:
        return 'mega'
    elif 'youtube.com' in url_lower or 'youtu.be' in url_lower:
        return 'youtube'
    elif 'soundcloud.com' in url_lower:
        return 'soundcloud'
    else:
        return 'other'

def verify_media_file(file_path, media_type='video'):
    try:
        cmd = ['ffprobe', '-v', 'error', '-show_format', '-show_streams', file_path]
        result = subprocess.run(cmd, check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        logger.info(f"Verified {media_type} file: {os.path.basename(file_path)}")
        return True
    except subprocess.CalledProcessError as e:
        logger.error(f"File verification failed for {file_path}: {e}")
        return False
    
def check_logs_exist():
    if os.path.exists(LOG_FOLDER):
        log_files = [f for f in os.listdir(LOG_FOLDER) if f.endswith('.log')]
        if log_files:
            print(f"Log files found in {LOG_FOLDER}:")
            for log_file in sorted(log_files):
                file_path = os.path.join(LOG_FOLDER, log_file)
                file_size = os.path.getsize(file_path)
                print(f"  - {log_file} ({file_size} bytes)")
            return True
        else:
            print(f"No log files found in {LOG_FOLDER}")
            return False
    else:
        print(f"Log folder does not exist: {LOG_FOLDER}")
        return False


import os
import subprocess
import time
import json
import shutil
from modules.helpers import (
    logger, create_folders, cleanup_temp_files,
    sanitize_filename, verify_media_file, detect_url_type
)
from modules.conversion import embed_metadata_with_ffmpeg
from modules.constants import TEMP_FOLDER

def get_metadata_from_url(url):
    try:
        cmd = [
            'yt-dlp',
            '--dump-json',
            '--no-warnings',
            '--no-playlist',
            url
        ]
        
        result = subprocess.run(cmd, capture_output=True, text=True, timeout=30, encoding='utf-8', errors='replace')
        if result.returncode == 0:
            metadata = json.loads(result.stdout)
            
            title = sanitize_filename(metadata.get('title', 'Unknown Title'))
            artist = sanitize_filename(metadata.get('uploader', metadata.get('channel', 'Unknown Artist')))
            album = sanitize_filename(metadata.get('album', metadata.get('playlist_title', 'Unknown Album')))
            
            logger.info(f"Extracted metadata: {title} by {artist}")
            
            return {
                'title': title,
                'artist': artist,
                'album': album,
                'year': str(metadata.get('upload_date', '')[:4]) if metadata.get('upload_date') else '',
                'genre': metadata.get('genre', ''),
                'duration': metadata.get('duration', 0)
            }
    except Exception as e:
        logger.warning(f"Could not extract metadata: {e}")
        print(f"Could not extract metadata: {e}")
    
    return None

def download_from_mega(url):
    create_folders()
    logger.info(f"Downloading from Mega: {url}")
    print(f"Downloading from Mega: {url}")
    
    try:
        # Try megatools-dl first, then fall back to megatools
        megatool_cmd = 'megatools-dl' if shutil.which('megatools-dl') else 'megatools'
        
        cmd = [
            megatool_cmd,
            '--path', TEMP_FOLDER,
            url
        ]
        
        logger.info("Running megatools download...")
        print("Running megatools download...")
        
        process = subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            universal_newlines=True,
            encoding='utf-8',
            errors='replace'
        )
        
        for line in process.stdout:
            if line.strip():
                print(line.strip(), flush=True)
        
        process.wait()
        
        if process.returncode != 0:
            raise subprocess.CalledProcessError(process.returncode, cmd)
        
        # Find downloaded file
        downloaded_files = [f for f in os.listdir(TEMP_FOLDER) 
                           if os.path.isfile(os.path.join(TEMP_FOLDER, f)) and not f.endswith(('.part', '.ytdl', '.temp'))]
        if not downloaded_files:
            raise Exception("Download completed but no file found")
        
        file_path = os.path.join(TEMP_FOLDER, max(
            downloaded_files,
            key=lambda x: os.path.getmtime(os.path.join(TEMP_FOLDER, x))
        ))
        
        logger.info(f"Download completed: {file_path}")
        print(f"Download completed: {file_path}")
        return file_path
        
    except subprocess.CalledProcessError as e:
        logger.error(f"Mega download failed: {e}")
        raise Exception(f"Mega download failed")
    except Exception as e:
        logger.error(f"Mega download error: {str(e)}")
        raise Exception(f"Mega download error: {str(e)}")

def download_with_ytdlp(url, media_type='video'):
    create_folders()
    cleanup_temp_files(TEMP_FOLDER)
    
    logger.info(f"Downloading with yt-dlp: {url}")
    print(f"Downloading with yt-dlp: {url}")
    
    metadata = None
    if media_type == 'music':
        logger.info("Extracting metadata...")
        print("Extracting metadata...")
        metadata = get_metadata_from_url(url)
        if metadata:
            logger.info(f"Found: {metadata['title']} by {metadata['artist']}")
            print(f"Found: {metadata['title']} by {metadata['artist']}")
    
    try:
        # Generate a safe filename template with ASCII fallback
        safe_template = os.path.join(TEMP_FOLDER, '%(title).80s.%(ext)s')
        
        if media_type == 'music':
            url_type = detect_url_type(url)
            
            if url_type == 'soundcloud':
                cmd = [
                    'yt-dlp',
                    '--extract-audio',
                    '--audio-format', 'mp3',
                    '--audio-quality', '0',  # Best quality
                    '--no-warnings',
                    '--no-playlist',
                    '--ignore-errors',
                    '--retries', '3',
                    '--fragment-retries', '3',
                    '--embed-metadata',  # Embed metadata in the file
                    '--add-metadata',    # Add metadata
                    '--restrict-filenames',  # Use ASCII-safe filenames
                    '-o', safe_template,
                    url
                ]
            else:
                # For other audio sources
                cmd = [
                    'yt-dlp',
                    '-f', 'bestaudio/best',
                    '--extract-audio',
                    '--audio-format', 'mp3',
                    '--audio-quality', '0',
                    '--no-warnings',
                    '--no-playlist',
                    '--ignore-errors',
                    '--retries', '3',
                    '--embed-metadata',  # Embed metadata in the file
                    '--add-metadata',    # Add metadata
                    '--restrict-filenames',  # Use ASCII-safe filenames
                    '-o', safe_template,
                    url
                ]
        else:
            # For video
            cmd = [
                'yt-dlp',
                '-f', 'best[height<=720]/best',  # Limit to 720p for Vita compatibility
                '--no-warnings',
                '--no-playlist',
                '--ignore-errors',
                '--retries', '3',
                '--restrict-filenames',  # Use ASCII-safe filenames
                '-o', safe_template,
                url
            ]
        
        logger.info("Running yt-dlp download...")
        print("Running yt-dlp download...")
        
        process = subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            universal_newlines=True,
            encoding='utf-8',
            errors='replace'
        )
        
        # Show real-time download progress
        for line in process.stdout:
            if line.strip():  # Only print non-empty lines
                if '[download]' in line or 'ERROR:' in line or 'WARNING:' in line:
                    print(line.strip(), flush=True)
        
        process.wait()
        
        if process.returncode != 0:
            raise subprocess.CalledProcessError(process.returncode, cmd)
        
        # Wait a moment for file system to sync
        time.sleep(2)
        
        # Clean up any partial files first
        cleanup_temp_files(TEMP_FOLDER)
        
        # Find downloaded file
        downloaded_files = []
        for f in os.listdir(TEMP_FOLDER):
            full_path = os.path.join(TEMP_FOLDER, f)
            if (os.path.isfile(full_path) and 
                not f.endswith(('.part', '.ytdl', '.temp')) and
                not f.startswith('.') and
                os.path.getsize(full_path) > 1024):  # File must be at least 1KB
                downloaded_files.append(f)
        
        if not downloaded_files:
            raise Exception("Download completed but no valid file found")
        
        # Get the most recently created file
        file_path = os.path.join(TEMP_FOLDER, max(
            downloaded_files,
            key=lambda x: os.path.getmtime(os.path.join(TEMP_FOLDER, x))
        ))
        
        if not verify_media_file(file_path, media_type if media_type == 'video' else 'audio'):
            raise Exception("Downloaded file appears to be corrupted")
        
        if media_type == 'music' and metadata:
            file_path = embed_metadata_with_ffmpeg(file_path, metadata)
        
        logger.info(f"Download completed: {file_path}")
        print(f"Download completed: {file_path}")
        return file_path
        
    except subprocess.TimeoutExpired:
        cleanup_temp_files(TEMP_FOLDER)
        logger.error("Download timed out (5 minutes)")
        raise Exception("Download timed out (5 minutes)")
    except subprocess.CalledProcessError as e:
        cleanup_temp_files(TEMP_FOLDER)
        logger.error(f"yt-dlp download failed with code {e.returncode}")
        raise Exception(f"yt-dlp download failed with code {e.returncode}")
    except Exception as e:
        cleanup_temp_files(TEMP_FOLDER)
        logger.error(f"yt-dlp download error: {str(e)}")
        raise Exception(f"yt-dlp download error: {str(e)}")

def download_media(url, media_type='video'):
    url_type = detect_url_type(url)
    logger.info(f"Downloading {media_type} from {url_type}: {url}")
    
    if url_type == 'mega':
        return download_from_mega(url)
    else:
        return download_with_ytdlp(url, media_type)